# frozen_string_literal: true

#--
# gravaty
# rubocop:disable Style/AsciiComments
# © 2013 Marco Bresciani
# rubocop:enable Style/AsciiComments
#
# This file is part of gravaty.
#
# gravaty is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# gravaty is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License
# along with gravaty. If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-FileCopyrightText: 2013 Marco Bresciani
#
# SPDX-License-Identifier: GPL-3.0-or-later
#++

require 'digest/md5'
require 'i18n'
require 'uri'

require_relative 'gravaty/application'
require_relative 'gravaty/parser'
require_relative 'gravaty/parsers/avatar'
require_relative 'gravaty/parsers/callback'
require_relative 'gravaty/parsers/default'
require_relative 'gravaty/parsers/force'
require_relative 'gravaty/parsers/format'
require_relative 'gravaty/parsers/pixelsize'
require_relative 'gravaty/parsers/rating'
require_relative 'gravaty/parsers/secure'
require_relative 'gravaty/parsers/type'

# Author:: {Marco Bresciani}[mailto:marcobresciani_1974@libero.it]
# rubocop:disable Style/AsciiComments
# Copyright:: Copyright © 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020  Marco Bresciani
# rubocop:enable Style/AsciiComments
# License:: GNU General Public License version 3
module Gravaty
  # Creates a new Gravaty described by the user's +email+. Throws a
  # <code>ArgumentError</code> exception if the supplied +email+ address
  # is nil or not valid according to RFC5322.
  #
  # Usage::
  #   - <tt>new_gravaty = Gravaty::gravatize email</tt>
  #   - <tt>new_gravaty = Gravaty::gravatize(email)</tt>
  # Params:: +email_address+, the user's email address (a syntactically
  # valid one).
  # Returns:: a +Gravaty+ object for the specified +email+ address.
  # Raises:: +ArgumentError+, if the supplied +email+ address is +nil+
  # or not valid according to RFC 5322.
  def self.gravatize(email_address)
    Gravaty.new email_address, parser
  end

  # ------------------------------ here starts the list of private methods

  # Provides the parameters' parsers object.
  def self.parser
    parsers = { avatar: Parsers::Avatar.new,
                callback: Parsers::Callback.new,
                default: Parsers::Default.new,
                force: Parsers::Force.new,
                format: Parsers::Format.new,
                pixelsize: Parsers::Pixelsize.new,
                rating: Parsers::Rating.new,
                secure: Parsers::Secure.new,
                type: Parsers::Type.new }

    Parser.new parsers
  end
end
