# frozen_string_literal: true

#--
# gravaty
# rubocop:disable Style/AsciiComments
# © 2012 Jon Maken
# rubocop:enable Style/AsciiComments
#
# This file is part of gravaty.
#
# gravaty is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# gravaty is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License
# along with gravaty. If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-FileCopyrightText: 2012 Jon Maken
#
# SPDX-License-Identifier: BSD-3-Clause
#++

module Gravaty
  module Utils
    # This module is a simple wrapper container for Peter R. Marreck's
    # gist.
    #
    # Author:: {Jon Maken}[mailto:jon.forums@gmail.com]
    # License:: 3-clause BSD
    # Revision:: 2012-03-25 23:01:19 -0600
    module Downloader
      require 'net/http'
      require 'net/ftp'
      require 'openssl'
      require 'fileutils'
      require 'tempfile'

      # !/usr/bin/env ruby
      #
      # An HTTP/HTTPS/FTP file downloader library/CLI based upon
      # MiniPortile's HTTP implementation.
      #
      # Author:: Jon Maken
      # License:: 3-clause BSD
      # Revision:: 2012-03-25 23:01:19 -0600
      class Downloader
        VERSION = '0.1.0'

        class << self
          attr_accessor :logger, :max_ca_verify_depth, :ftp_data_chunk_size
        end

        @logger = $stdout.binmode
        @max_ca_verify_depth = 5
        @ftp_data_chunk_size = 8192

        def self.download_file(url, full_path, count = 3)
          raise I18n.t('error.nil') if url.nil?
          return if File.exist?(full_path)

          http_download(url, full_path, count)
        end

        private_class_method def self.message(text)
                               @logger.print text
                               @logger.flush
                             end

        private_class_method def self.output(text = '')
                               @logger.puts text
                               @logger.flush
                             end

        private_class_method def self.http_download(url, full_path, count)
                               uri = URI.parse(url)

                               begin
                                 filename = File.basename(uri.path)

                                 if ENV['HTTP_PROXY']
                                   # protocol,
                                   userinfo, proxy_host, proxy_port = URI.split(ENV['HTTP_PROXY'])
                                   proxy_user, proxy_pass = userinfo.split(/:/) if userinfo
                                   http = Net::HTTP.new(uri.host, uri.port, proxy_host, proxy_port, proxy_user, proxy_pass)
                                 else
                                   http = Net::HTTP.new(uri.host, uri.port)
                                 end

                                 if uri.scheme.downcase == 'https'
                                   http.use_ssl = true
                                   if ENV['SSL_CERT_FILE']
                                     cert_file = ENV['SSL_CERT_FILE'].dup
                                     cert_file.gsub!(File::ALT_SEPARATOR, File::SEPARATOR) if File::ALT_SEPARATOR
                                   end
                                   if cert_file && File.exist?(cert_file)
                                     http.ca_file = cert_file
                                     http.verify_mode = OpenSSL::SSL::VERIFY_PEER
                                     http.verify_depth = @max_ca_verify_depth
                                   else
                                     raise <<~HOWTO_MESSAGE
                                                         To download using HTTPS you must first set the SSL_CERT_FILE
                                                         environment variable to the path of a valid CA certificate file.
                                                         A file of bundled public CA certs may be downloaded from:
                                       #{'                  '}
                                                            http://curl.haxx.se/ca/cacert.pem
                                       #{'                  '}
                                     HOWTO_MESSAGE
                                   end
                                 end

                                 http.request_get(uri) do |response|
                                   case response
                                   when Net::HTTPNotFound
                                     output '404 - Not Found'
                                     return false

                                   when Net::HTTPClientError
                                     output "Error: Client Error: #{response.inspect}"
                                     return false

                                   when Net::HTTPRedirection
                                     raise 'Too many redirections, halting.' if count <= 0

                                     url = response['location']
                                     return http_download(url, full_path, count - 1)

                                   when Net::HTTPOK
                                     temp_file = Tempfile
                                                   .new("download-#{filename}")
                                     temp_file.binmode
                                     temp_file << response.body

                                     output

                                     temp_file.close
                                     File.unlink full_path if File.exist?(full_path)
                                     FileUtils.mkdir_p File.dirname(full_path)
                                     FileUtils.mv temp_file.path, full_path, force: true
                                   else
                                     puts response
                                   end
                                 end
                               rescue StandardError => e
                                 File.unlink full_path if File.exist?(full_path)
                                 output "ERROR: #{e.message}"
                                 raise "Failed to download file: #{e.message}"
                               end
                             end
      end
    end
  end
end
