# frozen_string_literal: true

#--
# gravaty
# rubocop:disable Style/AsciiComments
# © 2013 Marco Bresciani
# rubocop:enable Style/AsciiComments
#
# This file is part of gravaty.
#
# gravaty is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# gravaty is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License
# along with gravaty. If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-FileCopyrightText: 2013 Marco Bresciani
#
# SPDX-License-Identifier: GPL-3.0-or-later
#++

require 'i18n'

require_relative 'raisers'

module Gravaty
  module Utils
    # This module contains the XML-RPC connector class.
    #
    # Author:: {Marco Bresciani}[mailto:marcobresciani_1974@libero.it]
    # rubocop:disable Style/AsciiComments
    # Copyright:: Copyright © 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020  Marco Bresciani
    # rubocop:enable Style/AsciiComments
    # License:: GNU General Public License version 3
    module RpcConnector
      # Creates a +RpcConnector+ described by the user's email +digest+
      # and a +password+. Throws a +ArgumentError+ if the supplied
      # +digest+ or the +password+ are not valid.
      #
      # Usage::
      #   - <tt>rpc_connector = RpcConnector.new digest, passowrd</tt>
      # Params::
      #   - +digest+, the user's email address MD5 digest.
      #   - +password+, the user's Gravatar account password.
      # Returns:: a +RcpConnector+ object for the specified email
      # +digest+.
      # Raises:: +ArgumentError+, if any of the supplied parameters are
      # not valid.
      class RpcConnector
        include Gravaty::Utils::Raisers

        def initialize(digest, password = nil)
          I18n.load_path =
            Dir[File.join(File.dirname(__FILE__),
                          '/../locales/', '*.yml')]

          raise ArgumentError, I18n.t('error.nil') if digest.nil? ||
                                                      digest.empty?
          raise ArgumentError, I18n.t('error.nil') if password.nil? ||
                                                      password.empty?

          @digest = digest
          @connector = XMLRPC::Client.new(RPC_URI, build_rpc_path, 443,
                                          nil, nil, nil, nil, true)

          @params = {}
          @params[:password] = password
        end

        def call(a_method = RPC_TEST_METHOD, params = {})
          raiser_downcase 'error.value', a_method, RPC_METHODS

          @connector.call a_method, @params.merge(params)
        end

        # ------------------------------ here starts the list of private methods

        private

        def build_rpc_path
          "/xmlrpc?user=#{@digest}"
        end
      end
    end
  end
end
